#Ermittelt die EXIF-Parameter einer Datei
#========================================

Param ([string]$ImagePath = "s:\desktop\test\D15A0663.JPG") #"s:\desktop\test\IMG_20190602_160848.jpg") 

if (!([System.IO.File]::Exists($ImagePath))) { "Bild existiert nicht"; return }
$ImageTypes = ".jpg", ".jpeg", ".png", ".tif", ".bmp"
$Extension = [System.IO.Path]::GetExtension($ImagePath).ToLower()
if (!($ImageTypes.Contains($Extension))) { "Kein erlaubtes Format"; return }

$CRLF = [char]13 + [char]10
function GetRatio64 { 
	Param ([byte[]]$b)
	$First = $b[0] + 256 * $b[1] + 65536 * $b[2] + 16777216 * $b[3] 
	$Second = $b[4] + 256 * $b[5] + 65536 * $b[6] + 16777216 * $b[7]  
	
	if ($First -gt 2147483648) { $First = $First - 4294967296 } 
	if ($Second -gt 2147483648) { $Second = $Second - 4294967296 }  
	if ($Second -eq 0) { $Second = 1 }  
	
	if (($First -eq 1) -and ($Second -ne 1)) { return ("1/" + $Second) } 
	else { return ($First / $Second) }
}
function GetNumber16 { 
	Param ([byte[]]$b)
	return $b[0] + 256 * $b[1];
}
function GetNumber32 { 
	Param ([byte[]]$b)
	return $b[0] + 256 * $b[1] + 65536 * $b[2] + 16777216 * $b[3]
}
function GetString { 
	Param ([byte[]]$b)
	$s = ''
	for ($i = 0; $i -lt $b.Length; $i++) { $s += [char]$b[$i] }
	return $s
}
$s = "Eigenschaften des Bildes $ImagePath"
$s += $CRLF + "".PadLeft($s.Length, '=') + $CRLF + $CRLF
$s = $CRLF + $s
$s += "Eigenschaften der Datei$CRLF$CRLF"
$FileInfo = [System.IO.FileInfo]::new($ImagePath)
$s += "Name: " + $FileInfo.Name + $CRLF
$s += "Verzeichnisname: " + $FileInfo.DirectoryName + $CRLF
$s += "Vollstndiger Name: " + $FileInfo.FullName + $CRLF
$s += "Erweiterung: " + $FileInfo.Extension + $CRLF
$s += "Nur Lesen: " + $FileInfo.IsReadOnly + $CRLF
$s += "Lnge: " + $FileInfo.Length + $CRLF
$s += "Mode: " + $FileInfo.Mode + $CRLF
$s += "Attribute: " + $FileInfo.Attributes + $CRLF
$s += "CreationTime: " + $FileInfo.CreationTime + $CRLF
$s += "LastAccessTime: " + $FileInfo.LastAccessTime + $CRLF
$s += "LastWriteTime: " + $FileInfo.LastWriteTime + $CRLF
$s += "CreationTimeUtc: " + $FileInfo.CreationTimeUtc + $CRLF
$s += "LastAccessTimeUtc: " + $FileInfo.LastAccessTimeUtc + $CRLF
$s += "LastWriteTimeUtc: " + $FileInfo.LastWriteTimeUtc + $CRLF
$s += $CRLF

Add-type -AssemblyName System.Drawing 
$s += "Eigenschaften des Bitmap-Objekts$CRLF$CRLF"
$Image = [System.Drawing.Bitmap]::new($ImagePath)
$Items = $Image.PropertyItems | Sort-Object -Property Id 
$PropertyIdList_sort = $Image.PropertyIdList | Sort-Object
$PropertyIdListHex_sort = $PropertyIdList_sort | ForEach-Object { "0x" + [Convert]::ToString($_, 16) }
#https://www.c-sharpcorner.com/uploadfile/mahesh/reading-metadata-of-images-in-gdi/
$Types = "", "byte", "byte[]", "int16", "int32", "rational64", "unused", "undef", "unused", "long", "rational"

$s += "Tag: " + $Image.Tag + $CRLF
$s += "PhysicalDimension: " + $Image.PhysicalDimension + $CRLF
$s += "Size: " + $Image.Size + $CRLF
$s += "Width: " + $Image.Width + $CRLF
$s += "Height: " + $Image.Height + $CRLF
$s += "HorizontalResolution: " + $Image.HorizontalResolution + $CRLF
$s += "VerticalResolution: " + $Image.VerticalResolution + $CRLF
$s += "Flags: " + $Image.Flags + $CRLF
$s += "RawFormat: " + $Image.RawFormat + $CRLF
$s += "PixelFormat: " + $Image.PixelFormat + $CRLF
$s += "Palette: " + $Image.Palette + $CRLF
$s += "FrameDimensionsList: " + $Image.FrameDimensionsList + $CRLF
$s += "PropertyIdListCount: " + $Image.PropertyIdList.Length + $CRLF
$s += "PropertyIdList: " + $PropertyIdList_sort + $CRLF
$s += "PropertyIdListHex: " + $PropertyIdListHex_sort + $CRLF
$s += "PropertyItems: $CRLF"
$s += "IdHex;Id;Type;TypeText;Length;Bytes;ExifName;ExifType;ExifValue" + $CRLF
$s += $CRLF
$s += "Eigenschaften der EXIF-Daten$CRLF$CRLF"
$ExifTable = Import-Csv -Path s:\desktop\exif\exif.csv -Header 'IDH', 'Title', 'Type', 'Comment' -Delimiter ";" | Select-Object *, Id
$ExifTable | ForEach-Object { $_.Id = [Convert]::ToInt64($_.IDH, 16) }
$ExifEntries = 0
$ExifEntriesNone = 0
foreach ($Item in $Items) { 
	$s1 = ''
	$s1 += "0x" + ([String][Convert]::ToString($Item.Id, 16)).PadLeft(4, '0') + ";" + [string]$Item.Id + ";" + [string]$Item.Type + ";" + $Types[$Item.Type] + ";" + [string]$Item.Len + ";" 
	for ($i = 0; $i -lt $Item.Len; $i++) {
		$s1 += [string]$Item.Value[$i] + " "
		if ($i -eq 10) { 
			if ($Item.Len -gt 10) {
				$s1 += "..."
				break 
			}
		}
	}
	$Id = $Item.Id
	$ExifItem = $ExifTable | Where-Object { $_.Id -eq $Id }
	$ExifName = $ExifItem.Title
	$ExifType = $ExifItem.Type
	if ($null -eq $ExifItem) {
		$ExifEntriesNone++
		$s1 += ";NOEXIF;;" 
		#switch ($Id) {
		#	20507 {
		#		$s1 += GetString $Item.Value
		#	}
		#	default {
		#	}
		#}
		$s1 += $CRLF 
	}
	else {
		$Value = $Item.Value
		switch -Wildcard ($ExifType) {
			'string*' { $s0 = GetString $Value }
			'int16*' { $s0 = GetNumber16 $Value } 
			'int32*' { $s0 = GetNumber32 $Value } 
			'rational64*' { $s0 = GetRatio64 $Value } 
			default { 
				$s0 = '' 
			}
		}
		$s1 += ";$ExifName;$ExifType;$s0$CRLF"
	}
	$s += $s1
	$ExifEntries++
}
$s
"EXIF-Eintrge: $ExifEntries, None: $ExifEntriesNone"
